# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import json
import pprint
import re  # noqa: F401

from typing import Optional
from pydantic import BaseModel, Field, StrictStr, ValidationError, validator
from synctera_client_v1.models.credit_application_patch import CreditApplicationPatch
from synctera_client_v1.models.restricted_application_patch import RestrictedApplicationPatch
from typing import Any, List
from pydantic import StrictStr, Field

APPLICATIONPATCH_ANY_OF_SCHEMAS = ["CreditApplicationPatch", "RestrictedApplicationPatch"]

class ApplicationPatch(BaseModel):
    """
    ApplicationPatch
    """

    # data type: CreditApplicationPatch
    anyof_schema_1_validator: Optional[CreditApplicationPatch] = None
    # data type: RestrictedApplicationPatch
    anyof_schema_2_validator: Optional[RestrictedApplicationPatch] = None
    actual_instance: Any
    any_of_schemas: List[str] = Field(APPLICATIONPATCH_ANY_OF_SCHEMAS, const=True)

    class Config:
        validate_assignment = True

    def __init__(self, *args, **kwargs):
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @validator('actual_instance')
    def actual_instance_must_validate_anyof(cls, v):
        instance = ApplicationPatch.construct()
        error_messages = []
        # validate data type: CreditApplicationPatch
        if not isinstance(v, CreditApplicationPatch):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CreditApplicationPatch`")
        else:
            return v

        # validate data type: RestrictedApplicationPatch
        if not isinstance(v, RestrictedApplicationPatch):
            error_messages.append(f"Error! Input type `{type(v)}` is not `RestrictedApplicationPatch`")
        else:
            return v

        if error_messages:
            # no match
            raise ValueError("No match found when setting the actual_instance in ApplicationPatch with anyOf schemas: CreditApplicationPatch, RestrictedApplicationPatch. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: dict) -> ApplicationPatch:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> ApplicationPatch:
        """Returns the object represented by the json string"""
        instance = ApplicationPatch.construct()
        error_messages = []
        # anyof_schema_1_validator: Optional[CreditApplicationPatch] = None
        try:
            instance.actual_instance = CreditApplicationPatch.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
             error_messages.append(str(e))
        # anyof_schema_2_validator: Optional[RestrictedApplicationPatch] = None
        try:
            instance.actual_instance = RestrictedApplicationPatch.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
             error_messages.append(str(e))

        if error_messages:
            # no match
            raise ValueError("No match found when deserializing the JSON string into ApplicationPatch with anyOf schemas: CreditApplicationPatch, RestrictedApplicationPatch. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        to_json = getattr(self.actual_instance, "to_json", None)
        if callable(to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> dict:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        to_json = getattr(self.actual_instance, "to_json", None)
        if callable(to_json):
            return self.actual_instance.to_dict()
        else:
            return json.dumps(self.actual_instance)

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.dict())




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from datetime import date

from pydantic import Field, StrictBool, StrictInt, StrictStr, conint, conlist

from typing import Optional

from synctera_client_v1.models.all_transaction_subtypes import AllTransactionSubtypes
from synctera_client_v1.models.paginated_transactions import PaginatedTransactions
from synctera_client_v1.models.transaction1 import Transaction1
from synctera_client_v1.models.transaction_statuses import TransactionStatuses
from synctera_client_v1.models.transaction_types import TransactionTypes

from synctera_client_v1.api_client import ApiClient
from synctera_client_v1.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class TransactionsBetaApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def get_transaction_by_id(self, id : Annotated[StrictStr, Field(..., description="Transaction ID")], **kwargs) -> Transaction1:  # noqa: E501
        """Get a transaction by ID  # noqa: E501

        > 🚧 Beta > This is a Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint.  Get a transaction by its uuid   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_transaction_by_id(id, async_req=True)
        >>> result = thread.get()

        :param id: Transaction ID (required)
        :type id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Transaction1
        """
        kwargs['_return_http_data_only'] = True
        return self.get_transaction_by_id_with_http_info(id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_transaction_by_id_with_http_info(self, id : Annotated[StrictStr, Field(..., description="Transaction ID")], **kwargs):  # noqa: E501
        """Get a transaction by ID  # noqa: E501

        > 🚧 Beta > This is a Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint.  Get a transaction by its uuid   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_transaction_by_id_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param id: Transaction ID (required)
        :type id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Transaction1, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_transaction_by_id" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "Transaction1",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '422': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/transactions/{id}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def list_transactions(self, posted_amount : Annotated[Optional[StrictInt], Field(description="Only return transactions equal to the given `posted_amount`")] = None, include_child_transactions : Annotated[Optional[StrictBool], Field(description="Include transactions from sub-accounts when listing transactions for a given account")] = None, amount : Annotated[Optional[StrictInt], Field(description="Only return transactions equal to the given `amount`")] = None, posted_date : Annotated[Optional[date], Field(description="Only return transactions matching the given `posted_date`")] = None, amount_lt : Annotated[Optional[StrictInt], Field(description="Only return transactions with `amount` less than `amount_lt`")] = None, type : Annotated[Optional[conlist(TransactionTypes)], Field(description="Only return transactions matching the given type")] = None, posted_amount_gte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `posted_amount` greater than or equal to `posted_amount_gte`")] = None, posted_amount_lt : Annotated[Optional[StrictInt], Field(description="Only return transactions with `posted_amount` less than `posted_amount_lt`")] = None, posted_amount_lte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `posted_amount` less than or equal to `posted_amount_lte`")] = None, status : Annotated[Optional[conlist(TransactionStatuses)], Field(description="Only return transactions with the given status or statuses")] = None, internal_account_id : Annotated[Optional[StrictStr], Field(description="Only return transactions matching the given `internal_account_id`")] = None, account_id : Annotated[Optional[StrictStr], Field(description="Account ID")] = None, pending_amount_lte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `pending_amount` less than or equal to `pending_amount_lte`")] = None, amount_gte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `amount` greater than or equal to `amount_gte`")] = None, group_id : Annotated[Optional[StrictStr], Field(description="Only return transactions matching the given `group_id`")] = None, pending_amount_gte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `pending_amount` greater than or equal to `pending_amount_gte`")] = None, exclude_jit_transactions : Annotated[Optional[StrictBool], Field(description="Hide \"JIT funding\" transactions from results")] = None, page_token : Optional[StrictStr] = None, card_id : Annotated[Optional[StrictStr], Field(description="Card ID")] = None, pending_amount : Annotated[Optional[StrictInt], Field(description="Only return transactions equal to the given `pending_amount`")] = None, account_source : Annotated[Optional[StrictStr], Field(description="Determines whether to return only transactions against customer accounts (`CUSTOMER`) vs internal accounts (`INTERNAL`). The default is to return all transactions if no `account_source` is specified.")] = None, posted_date_from : Annotated[Optional[date], Field(description="Only return transactions later than the given `posted_date`")] = None, posted_amount_gt : Annotated[Optional[StrictInt], Field(description="only return transactions with `posted_amount` greater than `posted_amount_gt`")] = None, pending_amount_gt : Annotated[Optional[StrictInt], Field(description="only return transactions with `pending_amount` greater than `pending_amount_gt`")] = None, limit : Optional[conint(strict=True, ge=1)] = None, amount_gt : Annotated[Optional[StrictInt], Field(description="Only return transactions with `amount` greater than `amount_gt`")] = None, posted_date_to : Annotated[Optional[date], Field(description="Only return transactions up to the given `posted_date`")] = None, amount_lte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `amount` less than or equal to `amount_lte`")] = None, subtype : Annotated[Optional[conlist(AllTransactionSubtypes)], Field(description="Only return transactions matching the given subtype")] = None, pending_amount_lt : Annotated[Optional[StrictInt], Field(description="Only return transactions with `pending_amount` less than `pending_amount_lt`")] = None, **kwargs) -> PaginatedTransactions:  # noqa: E501
        """List Transactions  # noqa: E501

        > 🚧 Beta > This is a Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint.  Get paginated list of transactions matching the provided filters   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_transactions(posted_amount, include_child_transactions, amount, posted_date, amount_lt, type, posted_amount_gte, posted_amount_lt, posted_amount_lte, status, internal_account_id, account_id, pending_amount_lte, amount_gte, group_id, pending_amount_gte, exclude_jit_transactions, page_token, card_id, pending_amount, account_source, posted_date_from, posted_amount_gt, pending_amount_gt, limit, amount_gt, posted_date_to, amount_lte, subtype, pending_amount_lt, async_req=True)
        >>> result = thread.get()

        :param posted_amount: Only return transactions equal to the given `posted_amount`
        :type posted_amount: int
        :param include_child_transactions: Include transactions from sub-accounts when listing transactions for a given account
        :type include_child_transactions: bool
        :param amount: Only return transactions equal to the given `amount`
        :type amount: int
        :param posted_date: Only return transactions matching the given `posted_date`
        :type posted_date: date
        :param amount_lt: Only return transactions with `amount` less than `amount_lt`
        :type amount_lt: int
        :param type: Only return transactions matching the given type
        :type type: List[TransactionTypes]
        :param posted_amount_gte: Only return transactions with `posted_amount` greater than or equal to `posted_amount_gte`
        :type posted_amount_gte: int
        :param posted_amount_lt: Only return transactions with `posted_amount` less than `posted_amount_lt`
        :type posted_amount_lt: int
        :param posted_amount_lte: Only return transactions with `posted_amount` less than or equal to `posted_amount_lte`
        :type posted_amount_lte: int
        :param status: Only return transactions with the given status or statuses
        :type status: List[TransactionStatuses]
        :param internal_account_id: Only return transactions matching the given `internal_account_id`
        :type internal_account_id: str
        :param account_id: Account ID
        :type account_id: str
        :param pending_amount_lte: Only return transactions with `pending_amount` less than or equal to `pending_amount_lte`
        :type pending_amount_lte: int
        :param amount_gte: Only return transactions with `amount` greater than or equal to `amount_gte`
        :type amount_gte: int
        :param group_id: Only return transactions matching the given `group_id`
        :type group_id: str
        :param pending_amount_gte: Only return transactions with `pending_amount` greater than or equal to `pending_amount_gte`
        :type pending_amount_gte: int
        :param exclude_jit_transactions: Hide \"JIT funding\" transactions from results
        :type exclude_jit_transactions: bool
        :param page_token:
        :type page_token: str
        :param card_id: Card ID
        :type card_id: str
        :param pending_amount: Only return transactions equal to the given `pending_amount`
        :type pending_amount: int
        :param account_source: Determines whether to return only transactions against customer accounts (`CUSTOMER`) vs internal accounts (`INTERNAL`). The default is to return all transactions if no `account_source` is specified.
        :type account_source: str
        :param posted_date_from: Only return transactions later than the given `posted_date`
        :type posted_date_from: date
        :param posted_amount_gt: only return transactions with `posted_amount` greater than `posted_amount_gt`
        :type posted_amount_gt: int
        :param pending_amount_gt: only return transactions with `pending_amount` greater than `pending_amount_gt`
        :type pending_amount_gt: int
        :param limit:
        :type limit: int
        :param amount_gt: Only return transactions with `amount` greater than `amount_gt`
        :type amount_gt: int
        :param posted_date_to: Only return transactions up to the given `posted_date`
        :type posted_date_to: date
        :param amount_lte: Only return transactions with `amount` less than or equal to `amount_lte`
        :type amount_lte: int
        :param subtype: Only return transactions matching the given subtype
        :type subtype: List[AllTransactionSubtypes]
        :param pending_amount_lt: Only return transactions with `pending_amount` less than `pending_amount_lt`
        :type pending_amount_lt: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PaginatedTransactions
        """
        kwargs['_return_http_data_only'] = True
        return self.list_transactions_with_http_info(posted_amount, include_child_transactions, amount, posted_date, amount_lt, type, posted_amount_gte, posted_amount_lt, posted_amount_lte, status, internal_account_id, account_id, pending_amount_lte, amount_gte, group_id, pending_amount_gte, exclude_jit_transactions, page_token, card_id, pending_amount, account_source, posted_date_from, posted_amount_gt, pending_amount_gt, limit, amount_gt, posted_date_to, amount_lte, subtype, pending_amount_lt, **kwargs)  # noqa: E501

    @validate_arguments
    def list_transactions_with_http_info(self, posted_amount : Annotated[Optional[StrictInt], Field(description="Only return transactions equal to the given `posted_amount`")] = None, include_child_transactions : Annotated[Optional[StrictBool], Field(description="Include transactions from sub-accounts when listing transactions for a given account")] = None, amount : Annotated[Optional[StrictInt], Field(description="Only return transactions equal to the given `amount`")] = None, posted_date : Annotated[Optional[date], Field(description="Only return transactions matching the given `posted_date`")] = None, amount_lt : Annotated[Optional[StrictInt], Field(description="Only return transactions with `amount` less than `amount_lt`")] = None, type : Annotated[Optional[conlist(TransactionTypes)], Field(description="Only return transactions matching the given type")] = None, posted_amount_gte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `posted_amount` greater than or equal to `posted_amount_gte`")] = None, posted_amount_lt : Annotated[Optional[StrictInt], Field(description="Only return transactions with `posted_amount` less than `posted_amount_lt`")] = None, posted_amount_lte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `posted_amount` less than or equal to `posted_amount_lte`")] = None, status : Annotated[Optional[conlist(TransactionStatuses)], Field(description="Only return transactions with the given status or statuses")] = None, internal_account_id : Annotated[Optional[StrictStr], Field(description="Only return transactions matching the given `internal_account_id`")] = None, account_id : Annotated[Optional[StrictStr], Field(description="Account ID")] = None, pending_amount_lte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `pending_amount` less than or equal to `pending_amount_lte`")] = None, amount_gte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `amount` greater than or equal to `amount_gte`")] = None, group_id : Annotated[Optional[StrictStr], Field(description="Only return transactions matching the given `group_id`")] = None, pending_amount_gte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `pending_amount` greater than or equal to `pending_amount_gte`")] = None, exclude_jit_transactions : Annotated[Optional[StrictBool], Field(description="Hide \"JIT funding\" transactions from results")] = None, page_token : Optional[StrictStr] = None, card_id : Annotated[Optional[StrictStr], Field(description="Card ID")] = None, pending_amount : Annotated[Optional[StrictInt], Field(description="Only return transactions equal to the given `pending_amount`")] = None, account_source : Annotated[Optional[StrictStr], Field(description="Determines whether to return only transactions against customer accounts (`CUSTOMER`) vs internal accounts (`INTERNAL`). The default is to return all transactions if no `account_source` is specified.")] = None, posted_date_from : Annotated[Optional[date], Field(description="Only return transactions later than the given `posted_date`")] = None, posted_amount_gt : Annotated[Optional[StrictInt], Field(description="only return transactions with `posted_amount` greater than `posted_amount_gt`")] = None, pending_amount_gt : Annotated[Optional[StrictInt], Field(description="only return transactions with `pending_amount` greater than `pending_amount_gt`")] = None, limit : Optional[conint(strict=True, ge=1)] = None, amount_gt : Annotated[Optional[StrictInt], Field(description="Only return transactions with `amount` greater than `amount_gt`")] = None, posted_date_to : Annotated[Optional[date], Field(description="Only return transactions up to the given `posted_date`")] = None, amount_lte : Annotated[Optional[StrictInt], Field(description="Only return transactions with `amount` less than or equal to `amount_lte`")] = None, subtype : Annotated[Optional[conlist(AllTransactionSubtypes)], Field(description="Only return transactions matching the given subtype")] = None, pending_amount_lt : Annotated[Optional[StrictInt], Field(description="Only return transactions with `pending_amount` less than `pending_amount_lt`")] = None, **kwargs):  # noqa: E501
        """List Transactions  # noqa: E501

        > 🚧 Beta > This is a Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint.  Get paginated list of transactions matching the provided filters   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_transactions_with_http_info(posted_amount, include_child_transactions, amount, posted_date, amount_lt, type, posted_amount_gte, posted_amount_lt, posted_amount_lte, status, internal_account_id, account_id, pending_amount_lte, amount_gte, group_id, pending_amount_gte, exclude_jit_transactions, page_token, card_id, pending_amount, account_source, posted_date_from, posted_amount_gt, pending_amount_gt, limit, amount_gt, posted_date_to, amount_lte, subtype, pending_amount_lt, async_req=True)
        >>> result = thread.get()

        :param posted_amount: Only return transactions equal to the given `posted_amount`
        :type posted_amount: int
        :param include_child_transactions: Include transactions from sub-accounts when listing transactions for a given account
        :type include_child_transactions: bool
        :param amount: Only return transactions equal to the given `amount`
        :type amount: int
        :param posted_date: Only return transactions matching the given `posted_date`
        :type posted_date: date
        :param amount_lt: Only return transactions with `amount` less than `amount_lt`
        :type amount_lt: int
        :param type: Only return transactions matching the given type
        :type type: List[TransactionTypes]
        :param posted_amount_gte: Only return transactions with `posted_amount` greater than or equal to `posted_amount_gte`
        :type posted_amount_gte: int
        :param posted_amount_lt: Only return transactions with `posted_amount` less than `posted_amount_lt`
        :type posted_amount_lt: int
        :param posted_amount_lte: Only return transactions with `posted_amount` less than or equal to `posted_amount_lte`
        :type posted_amount_lte: int
        :param status: Only return transactions with the given status or statuses
        :type status: List[TransactionStatuses]
        :param internal_account_id: Only return transactions matching the given `internal_account_id`
        :type internal_account_id: str
        :param account_id: Account ID
        :type account_id: str
        :param pending_amount_lte: Only return transactions with `pending_amount` less than or equal to `pending_amount_lte`
        :type pending_amount_lte: int
        :param amount_gte: Only return transactions with `amount` greater than or equal to `amount_gte`
        :type amount_gte: int
        :param group_id: Only return transactions matching the given `group_id`
        :type group_id: str
        :param pending_amount_gte: Only return transactions with `pending_amount` greater than or equal to `pending_amount_gte`
        :type pending_amount_gte: int
        :param exclude_jit_transactions: Hide \"JIT funding\" transactions from results
        :type exclude_jit_transactions: bool
        :param page_token:
        :type page_token: str
        :param card_id: Card ID
        :type card_id: str
        :param pending_amount: Only return transactions equal to the given `pending_amount`
        :type pending_amount: int
        :param account_source: Determines whether to return only transactions against customer accounts (`CUSTOMER`) vs internal accounts (`INTERNAL`). The default is to return all transactions if no `account_source` is specified.
        :type account_source: str
        :param posted_date_from: Only return transactions later than the given `posted_date`
        :type posted_date_from: date
        :param posted_amount_gt: only return transactions with `posted_amount` greater than `posted_amount_gt`
        :type posted_amount_gt: int
        :param pending_amount_gt: only return transactions with `pending_amount` greater than `pending_amount_gt`
        :type pending_amount_gt: int
        :param limit:
        :type limit: int
        :param amount_gt: Only return transactions with `amount` greater than `amount_gt`
        :type amount_gt: int
        :param posted_date_to: Only return transactions up to the given `posted_date`
        :type posted_date_to: date
        :param amount_lte: Only return transactions with `amount` less than or equal to `amount_lte`
        :type amount_lte: int
        :param subtype: Only return transactions matching the given subtype
        :type subtype: List[AllTransactionSubtypes]
        :param pending_amount_lt: Only return transactions with `pending_amount` less than `pending_amount_lt`
        :type pending_amount_lt: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PaginatedTransactions, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'posted_amount',
            'include_child_transactions',
            'amount',
            'posted_date',
            'amount_lt',
            'type',
            'posted_amount_gte',
            'posted_amount_lt',
            'posted_amount_lte',
            'status',
            'internal_account_id',
            'account_id',
            'pending_amount_lte',
            'amount_gte',
            'group_id',
            'pending_amount_gte',
            'exclude_jit_transactions',
            'page_token',
            'card_id',
            'pending_amount',
            'account_source',
            'posted_date_from',
            'posted_amount_gt',
            'pending_amount_gt',
            'limit',
            'amount_gt',
            'posted_date_to',
            'amount_lte',
            'subtype',
            'pending_amount_lt'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_transactions" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('posted_amount') is not None:  # noqa: E501
            _query_params.append(('posted_amount', _params['posted_amount']))

        if _params.get('include_child_transactions') is not None:  # noqa: E501
            _query_params.append(('include_child_transactions', _params['include_child_transactions']))

        if _params.get('amount') is not None:  # noqa: E501
            _query_params.append(('amount', _params['amount']))

        if _params.get('posted_date') is not None:  # noqa: E501
            if isinstance(_params['posted_date'], datetime):
                _query_parame.append(('posted_date', _params['posted_date'].strftime(self.api_client.configuration.date_format)))
            else:
                _query_params.append(('posted_date', _params['posted_date']))

        if _params.get('amount_lt') is not None:  # noqa: E501
            _query_params.append(('amount_lt', _params['amount_lt']))

        if _params.get('type') is not None:  # noqa: E501
            _query_params.append(('type', _params['type']))
            _collection_formats['type'] = 'csv'

        if _params.get('posted_amount_gte') is not None:  # noqa: E501
            _query_params.append(('posted_amount_gte', _params['posted_amount_gte']))

        if _params.get('posted_amount_lt') is not None:  # noqa: E501
            _query_params.append(('posted_amount_lt', _params['posted_amount_lt']))

        if _params.get('posted_amount_lte') is not None:  # noqa: E501
            _query_params.append(('posted_amount_lte', _params['posted_amount_lte']))

        if _params.get('status') is not None:  # noqa: E501
            _query_params.append(('status', _params['status']))
            _collection_formats['status'] = 'csv'

        if _params.get('internal_account_id') is not None:  # noqa: E501
            _query_params.append(('internal_account_id', _params['internal_account_id']))

        if _params.get('account_id') is not None:  # noqa: E501
            _query_params.append(('account_id', _params['account_id']))

        if _params.get('pending_amount_lte') is not None:  # noqa: E501
            _query_params.append(('pending_amount_lte', _params['pending_amount_lte']))

        if _params.get('amount_gte') is not None:  # noqa: E501
            _query_params.append(('amount_gte', _params['amount_gte']))

        if _params.get('group_id') is not None:  # noqa: E501
            _query_params.append(('group_id', _params['group_id']))

        if _params.get('pending_amount_gte') is not None:  # noqa: E501
            _query_params.append(('pending_amount_gte', _params['pending_amount_gte']))

        if _params.get('exclude_jit_transactions') is not None:  # noqa: E501
            _query_params.append(('exclude_jit_transactions', _params['exclude_jit_transactions']))

        if _params.get('page_token') is not None:  # noqa: E501
            _query_params.append(('page_token', _params['page_token']))

        if _params.get('card_id') is not None:  # noqa: E501
            _query_params.append(('card_id', _params['card_id']))

        if _params.get('pending_amount') is not None:  # noqa: E501
            _query_params.append(('pending_amount', _params['pending_amount']))

        if _params.get('account_source') is not None:  # noqa: E501
            _query_params.append(('account_source', _params['account_source'].value))

        if _params.get('posted_date_from') is not None:  # noqa: E501
            if isinstance(_params['posted_date_from'], datetime):
                _query_parame.append(('posted_date_from', _params['posted_date_from'].strftime(self.api_client.configuration.date_format)))
            else:
                _query_params.append(('posted_date_from', _params['posted_date_from']))

        if _params.get('posted_amount_gt') is not None:  # noqa: E501
            _query_params.append(('posted_amount_gt', _params['posted_amount_gt']))

        if _params.get('pending_amount_gt') is not None:  # noqa: E501
            _query_params.append(('pending_amount_gt', _params['pending_amount_gt']))

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('amount_gt') is not None:  # noqa: E501
            _query_params.append(('amount_gt', _params['amount_gt']))

        if _params.get('posted_date_to') is not None:  # noqa: E501
            if isinstance(_params['posted_date_to'], datetime):
                _query_parame.append(('posted_date_to', _params['posted_date_to'].strftime(self.api_client.configuration.date_format)))
            else:
                _query_params.append(('posted_date_to', _params['posted_date_to']))

        if _params.get('amount_lte') is not None:  # noqa: E501
            _query_params.append(('amount_lte', _params['amount_lte']))

        if _params.get('subtype') is not None:  # noqa: E501
            _query_params.append(('subtype', _params['subtype']))
            _collection_formats['subtype'] = 'csv'

        if _params.get('pending_amount_lt') is not None:  # noqa: E501
            _query_params.append(('pending_amount_lt', _params['pending_amount_lt']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "PaginatedTransactions",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '422': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/transactions', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json

from datetime import date, datetime
from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr, validator
from synctera_client_v1.models.base_transaction_decline import BaseTransactionDecline
from synctera_client_v1.models.enhanced_transaction_data1 import EnhancedTransactionData1
from synctera_client_v1.models.fraud_risk_data import FraudRiskData
from synctera_client_v1.models.transaction_statuses import TransactionStatuses
from synctera_client_v1.models.transaction_types import TransactionTypes

class BaseTransaction(BaseModel):
    """
    BaseTransaction
    """
    account_id: Optional[StrictStr] = Field(None, description="The account uuid associated with the transaction. `account_id` and `internal_account_id` are mutually exclusive")
    amount: StrictInt = Field(..., description="The total amount of the transaction including both pending and already posted amounts. The value is represented as the smallest denomination of the applicable currency.")
    creation_time: datetime = Field(..., description="The exact time the transaction was recorded in the ledger")
    currency: StrictStr = Field(..., description="ISO 4217 alphabetic currency code of the transfer amount")
    customer_id: Optional[StrictStr] = Field(None, description="The uuid of the customer that initiated the transaction (if any) ")
    dc_sign: StrictStr = Field(..., description="The `dc_sign` represents the direction money was moved. A value of `DEBIT` is money moving out of an account, a value of `CREDIT` is money moving into an account")
    decline: Optional[BaseTransactionDecline] = None
    description: Optional[StrictStr] = Field(None, description="A human-friendly description of the transaction, provided by the integrator")
    effective_date: date = Field(..., description="The effective date of the transaction. This usually aligns with network settlement date, which differs between transaction types. The effective date is also used to determine effective daily balances for the purposes of interest calculation.")
    enhanced_transaction: Optional[EnhancedTransactionData1] = None
    force_post: StrictBool = Field(..., description="Determines whether or not a transaction or auth was \"forced\" or not. A forced transaction skips any account balance checks")
    fraud_risk: Optional[FraudRiskData] = None
    group_id: StrictStr = Field(..., description="The group id of the transaction. Every transaction in the ledger is one entry in a double-entry system and the primary and offset transactions share the same `group_id`")
    hold_expiration_time: Optional[datetime] = Field(None, description="The date and time any pending amount is expected to be released back to the account.")
    id: StrictStr = Field(..., description="The unique identifier of the transaction")
    internal_account_id: Optional[StrictStr] = Field(None, description="The internal account uuid associated with the transaction. `account_id` and `internal_account_id` are mutually exclusive")
    last_updated_time: datetime = Field(..., description="The date and time the transaction was last modified")
    metadata: Optional[Dict[str, Any]] = Field(None, description="an unstructured json blob representing additional transaction information supplied by the integrator.")
    partial_hold: StrictBool = Field(..., description="Determines whether or not the funds on hold were the result of a partial auth or not. If `true` the `pending_amount` of the transaction will be less than the requested amount. This is primarily used for certain types of card transactions.")
    pending_amount: StrictInt = Field(..., description="The amount amount of the transaction currently authorized or on hold")
    posted_amount: StrictInt = Field(..., description="The amount of the transaction that has been fully posted to the account")
    posted_date: Optional[date] = Field(None, description="The date the transaction was posted (based on the bank calendar and end-of-day). For transaction with multiple postings, this is the date of the earliest posting. This will be omitted for transactions with a `posted_amount` of `0`.")
    reference_id: Optional[StrictStr] = Field(None, description="An external ID provided by the payment network to represent this transaction. This is not guaranteed to be globally unique. This will always be omitted for internal transfers.")
    status: TransactionStatuses = Field(...)
    system_description: Optional[StrictStr] = Field(None, description="A human-friendly description of the transaction, provided by the Synctera platform")
    transaction_time: datetime = Field(..., description="The time the transaction occurred. In most cases this will be roughly identical to creation_time, but it can differ in some situations if the payment doesn't appear in the ledger in real-time.")
    type: TransactionTypes = Field(...)
    __properties = ["account_id", "amount", "creation_time", "currency", "customer_id", "dc_sign", "decline", "description", "effective_date", "enhanced_transaction", "force_post", "fraud_risk", "group_id", "hold_expiration_time", "id", "internal_account_id", "last_updated_time", "metadata", "partial_hold", "pending_amount", "posted_amount", "posted_date", "reference_id", "status", "system_description", "transaction_time", "type"]

    @validator('dc_sign')
    def dc_sign_validate_enum(cls, v):
        if v not in ('CREDIT', 'DEBIT'):
            raise ValueError("must be one of enum values ('CREDIT', 'DEBIT')")
        return v

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BaseTransaction:
        """Create an instance of BaseTransaction from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of decline
        if self.decline:
            _dict['decline'] = self.decline.to_dict()
        # override the default output from pydantic by calling `to_dict()` of enhanced_transaction
        if self.enhanced_transaction:
            _dict['enhanced_transaction'] = self.enhanced_transaction.to_dict()
        # override the default output from pydantic by calling `to_dict()` of fraud_risk
        if self.fraud_risk:
            _dict['fraud_risk'] = self.fraud_risk.to_dict()
        # set to None if metadata (nullable) is None
        # and __fields_set__ contains the field
        if self.metadata is None and "metadata" in self.__fields_set__:
            _dict['metadata'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BaseTransaction:
        """Create an instance of BaseTransaction from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return BaseTransaction.parse_obj(obj)

        _obj = BaseTransaction.parse_obj({
            "account_id": obj.get("account_id"),
            "amount": obj.get("amount"),
            "creation_time": obj.get("creation_time"),
            "currency": obj.get("currency"),
            "customer_id": obj.get("customer_id"),
            "dc_sign": obj.get("dc_sign"),
            "decline": BaseTransactionDecline.from_dict(obj.get("decline")) if obj.get("decline") is not None else None,
            "description": obj.get("description"),
            "effective_date": obj.get("effective_date"),
            "enhanced_transaction": EnhancedTransactionData1.from_dict(obj.get("enhanced_transaction")) if obj.get("enhanced_transaction") is not None else None,
            "force_post": obj.get("force_post"),
            "fraud_risk": FraudRiskData.from_dict(obj.get("fraud_risk")) if obj.get("fraud_risk") is not None else None,
            "group_id": obj.get("group_id"),
            "hold_expiration_time": obj.get("hold_expiration_time"),
            "id": obj.get("id"),
            "internal_account_id": obj.get("internal_account_id"),
            "last_updated_time": obj.get("last_updated_time"),
            "metadata": obj.get("metadata"),
            "partial_hold": obj.get("partial_hold"),
            "pending_amount": obj.get("pending_amount"),
            "posted_amount": obj.get("posted_amount"),
            "posted_date": obj.get("posted_date"),
            "reference_id": obj.get("reference_id"),
            "status": obj.get("status"),
            "system_description": obj.get("system_description"),
            "transaction_time": obj.get("transaction_time"),
            "type": obj.get("type")
        })
        return _obj




# flake8: noqa

# import apis into api package
from synctera_client_v1.api.applications_beta_api import ApplicationsBetaApi
from synctera_client_v1.api.card_transaction_simulations_beta_api import CardTransactionSimulationsBetaApi
from synctera_client_v1.api.card_webhook_simulations_beta_api import CardWebhookSimulationsBetaApi
from synctera_client_v1.api.cards_beta_api import CardsBetaApi
from synctera_client_v1.api.digital_wallet_tokens_beta_api import DigitalWalletTokensBetaApi
from synctera_client_v1.api.external_cards_beta_api import ExternalCardsBetaApi
from synctera_client_v1.api.spend_controls_beta_api import SpendControlsBetaApi
from synctera_client_v1.api.statements_api import StatementsApi
from synctera_client_v1.api.transactions_beta_api import TransactionsBetaApi
from synctera_client_v1.api.webhooks_api import WebhooksApi




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json



from pydantic import BaseModel, Field, StrictInt, StrictStr
from synctera_client_v1.models.dc_sign import DcSign

class TransactionLineV0(BaseModel):
    """
    TransactionLineV0
    """
    account_id: StrictStr = Field(..., description="The account uuid associated with this transaction line")
    account_no: StrictStr = Field(..., description="The account number associated with this transaction line")
    amount: StrictInt = Field(..., description="The amount (in ISO 4217 minor currency units) of the transaction")
    currency: StrictStr = Field(..., description="ISO 4217 alphabetic currency code of the transfer amount")
    dc_sign: DcSign = Field(...)
    uuid: StrictStr = Field(...)
    __properties = ["account_id", "account_no", "amount", "currency", "dc_sign", "uuid"]

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TransactionLineV0:
        """Create an instance of TransactionLineV0 from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TransactionLineV0:
        """Create an instance of TransactionLineV0 from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return TransactionLineV0.parse_obj(obj)

        _obj = TransactionLineV0.parse_obj({
            "account_id": obj.get("account_id"),
            "account_no": obj.get("account_no"),
            "amount": obj.get("amount"),
            "currency": obj.get("currency"),
            "dc_sign": obj.get("dc_sign"),
            "uuid": obj.get("uuid")
        })
        return _obj




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, constr, validator
from synctera_client_v1.models.create_transfer_type import CreateTransferType
from synctera_client_v1.models.merchant import Merchant

class CreateTransferResponsePull(BaseModel):
    """
    CreateTransferResponsePull
    """
    three_ds_id: Optional[StrictStr] = Field(None, description="Unique identifier of an External Card Transfer 3-D Secure Authorization - conditionally required according to your program's 3DS policy")
    type: CreateTransferType = Field(...)
    account_id: StrictStr = Field(..., description="The ID of the Synctera account into which or from which funds were moved")
    amount: StrictInt = Field(..., description="Amount of the transfer in cents")
    creation_time: datetime = Field(...)
    currency: constr(strict=True, max_length=3, min_length=3) = Field(..., description="ISO 4217  Alpha-3 currency code")
    customer_id: StrictStr = Field(..., description="The ID of the customer to whom the card belongs")
    external_card_id: StrictStr = Field(..., description="The ID of the external card from/to which the transfer was initiated/received")
    id: StrictStr = Field(..., description="The ID of the transfer")
    last_updated_time: datetime = Field(...)
    merchant: Merchant = Field(...)
    status: StrictStr = Field(..., description="The status of the transfer")
    tenant: StrictStr = Field(..., description="The id of the tenant containing the resource. ")
    transaction_id: Optional[StrictStr] = Field(None, description="The transaction ID")
    __properties = ["three_ds_id", "type", "account_id", "amount", "creation_time", "currency", "customer_id", "external_card_id", "id", "last_updated_time", "merchant", "status", "tenant", "transaction_id"]

    @validator('status')
    def status_validate_enum(cls, v):
        if v not in ('CANCELED', 'DECLINED', 'PENDING', 'SUCCEEDED', 'UNKNOWN'):
            raise ValueError("must be one of enum values ('CANCELED', 'DECLINED', 'PENDING', 'SUCCEEDED', 'UNKNOWN')")
        return v

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateTransferResponsePull:
        """Create an instance of CreateTransferResponsePull from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of merchant
        if self.merchant:
            _dict['merchant'] = self.merchant.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateTransferResponsePull:
        """Create an instance of CreateTransferResponsePull from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return CreateTransferResponsePull.parse_obj(obj)

        _obj = CreateTransferResponsePull.parse_obj({
            "three_ds_id": obj.get("three_ds_id"),
            "type": obj.get("type"),
            "account_id": obj.get("account_id"),
            "amount": obj.get("amount"),
            "creation_time": obj.get("creation_time"),
            "currency": obj.get("currency"),
            "customer_id": obj.get("customer_id"),
            "external_card_id": obj.get("external_card_id"),
            "id": obj.get("id"),
            "last_updated_time": obj.get("last_updated_time"),
            "merchant": Merchant.from_dict(obj.get("merchant")) if obj.get("merchant") is not None else None,
            "status": obj.get("status"),
            "tenant": obj.get("tenant"),
            "transaction_id": obj.get("transaction_id")
        })
        return _obj




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json


from typing import Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr

class EnhancedTransactionData1EnhancedRawInner(BaseModel):
    """
    EnhancedTransactionData1EnhancedRawInner
    """
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The monetary amount of the transaction")
    categorized_by: Optional[StrictInt] = Field(None, description="The method used to detrmine the category")
    category: Optional[StrictStr] = Field(None, description="The category of the transaction")
    category_guid: Optional[StrictStr] = Field(None, description="The unique identifier for the category")
    described_by: Optional[StrictInt] = Field(None, description="The method used to describe the transaction")
    description: Optional[StrictStr] = Field(None, description="A human-readable version of `original_description`")
    extended_transaction_type: Optional[StrictStr] = Field(None, description="The transaction type assigned by the partner")
    id: Optional[StrictInt] = Field(None, description="The unique partner-defined identifier for the transaction")
    is_bill_pay: Optional[StrictBool] = Field(None, description="Whether the transaction represents a bill payment")
    is_direct_deposit: Optional[StrictBool] = Field(None, description="Whether or not the transaction represents a direct deposit")
    is_expense: Optional[StrictBool] = Field(None, description="Whether or not the transaction represents an expense")
    is_fee: Optional[StrictBool] = Field(None, description="Whether or not the transaction represents a fee")
    is_income: Optional[StrictBool] = Field(None, description="Whether or not the transaction represents income")
    is_international: Optional[StrictBool] = Field(None, description="Whether or not the transaction is international")
    is_overdraft_fee: Optional[StrictBool] = Field(None, description="Whether or not the transaction is an overdraft fee")
    is_payroll_advance: Optional[StrictBool] = Field(None, description="Whether or not the transaction is a payroll advance")
    is_subscription: Optional[StrictBool] = Field(None, description="Whether or not the transaction is a subscription")
    memo: Optional[StrictStr] = Field(None, description="Additional descriptiive information about the transaction")
    merchant_category_code: Optional[StrictInt] = Field(None, description="The ISO 18245 category code for the transaction")
    merchant_guid: Optional[StrictStr] = Field(None, description="The unique identifier for the merchant")
    merchant_location_guid: Optional[StrictStr] = Field(None, description="The unique identifier for the merchant location")
    original_description: Optional[StrictStr] = Field(None, description="The original description for the transaction")
    type: Optional[StrictStr] = Field(None, description="The type of the transsaction. This will be either `CREDIT` or `DEBIT`")
    __properties = ["amount", "categorized_by", "category", "category_guid", "described_by", "description", "extended_transaction_type", "id", "is_bill_pay", "is_direct_deposit", "is_expense", "is_fee", "is_income", "is_international", "is_overdraft_fee", "is_payroll_advance", "is_subscription", "memo", "merchant_category_code", "merchant_guid", "merchant_location_guid", "original_description", "type"]

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> EnhancedTransactionData1EnhancedRawInner:
        """Create an instance of EnhancedTransactionData1EnhancedRawInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> EnhancedTransactionData1EnhancedRawInner:
        """Create an instance of EnhancedTransactionData1EnhancedRawInner from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return EnhancedTransactionData1EnhancedRawInner.parse_obj(obj)

        _obj = EnhancedTransactionData1EnhancedRawInner.parse_obj({
            "amount": obj.get("amount"),
            "categorized_by": obj.get("categorized_by"),
            "category": obj.get("category"),
            "category_guid": obj.get("category_guid"),
            "described_by": obj.get("described_by"),
            "description": obj.get("description"),
            "extended_transaction_type": obj.get("extended_transaction_type"),
            "id": obj.get("id"),
            "is_bill_pay": obj.get("is_bill_pay"),
            "is_direct_deposit": obj.get("is_direct_deposit"),
            "is_expense": obj.get("is_expense"),
            "is_fee": obj.get("is_fee"),
            "is_income": obj.get("is_income"),
            "is_international": obj.get("is_international"),
            "is_overdraft_fee": obj.get("is_overdraft_fee"),
            "is_payroll_advance": obj.get("is_payroll_advance"),
            "is_subscription": obj.get("is_subscription"),
            "memo": obj.get("memo"),
            "merchant_category_code": obj.get("merchant_category_code"),
            "merchant_guid": obj.get("merchant_guid"),
            "merchant_location_guid": obj.get("merchant_location_guid"),
            "original_description": obj.get("original_description"),
            "type": obj.get("type")
        })
        return _obj




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json


from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictInt, StrictStr, confloat, conint, conlist, validator
from synctera_client_v1.models.income import Income

class FinancialStatus(BaseModel):
    """
    Financial status of the applicant. Only applicable for customers. 
    """
    debt_to_income_ratio: Optional[Union[confloat(le=100, ge=0, strict=True), conint(le=100, ge=0, strict=True)]] = Field(None, description="Debt-to-income ratio. It is between 0% and 100% and allows for decimals")
    employment_status: Optional[StrictStr] = Field(None, description="Employment status of the applicant")
    income: Optional[conlist(Income)] = None
    monthly_rent_mortgage: Optional[StrictInt] = Field(None, description="Monthly rent/mortgage in cents, i.e. $2200 is represented as 220000")
    __properties = ["debt_to_income_ratio", "employment_status", "income", "monthly_rent_mortgage"]

    @validator('employment_status')
    def employment_status_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ('EMPLOYED', 'OTHER', 'SELF-EMPLOYED'):
            raise ValueError("must be one of enum values ('EMPLOYED', 'OTHER', 'SELF-EMPLOYED')")
        return v

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FinancialStatus:
        """Create an instance of FinancialStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in income (list)
        _items = []
        if self.income:
            for _item in self.income:
                if _item:
                    _items.append(_item.to_dict())
            _dict['income'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FinancialStatus:
        """Create an instance of FinancialStatus from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return FinancialStatus.parse_obj(obj)

        _obj = FinancialStatus.parse_obj({
            "debt_to_income_ratio": obj.get("debt_to_income_ratio"),
            "employment_status": obj.get("employment_status"),
            "income": [Income.from_dict(_item) for _item in obj.get("income")] if obj.get("income") is not None else None,
            "monthly_rent_mortgage": obj.get("monthly_rent_mortgage")
        })
        return _obj




# coding: utf-8

# flake8: noqa

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


__version__ = "1.41.3"

# import apis into sdk package
from synctera_client_v1.api.applications_beta_api import ApplicationsBetaApi
from synctera_client_v1.api.card_transaction_simulations_beta_api import CardTransactionSimulationsBetaApi
from synctera_client_v1.api.card_webhook_simulations_beta_api import CardWebhookSimulationsBetaApi
from synctera_client_v1.api.cards_beta_api import CardsBetaApi
from synctera_client_v1.api.digital_wallet_tokens_beta_api import DigitalWalletTokensBetaApi
from synctera_client_v1.api.external_cards_beta_api import ExternalCardsBetaApi
from synctera_client_v1.api.spend_controls_beta_api import SpendControlsBetaApi
from synctera_client_v1.api.statements_api import StatementsApi
from synctera_client_v1.api.transactions_beta_api import TransactionsBetaApi
from synctera_client_v1.api.webhooks_api import WebhooksApi

# import ApiClient
from synctera_client_v1.api_response import ApiResponse
from synctera_client_v1.api_client import ApiClient
from synctera_client_v1.configuration import Configuration
from synctera_client_v1.exceptions import OpenApiException
from synctera_client_v1.exceptions import ApiTypeError
from synctera_client_v1.exceptions import ApiValueError
from synctera_client_v1.exceptions import ApiKeyError
from synctera_client_v1.exceptions import ApiAttributeError
from synctera_client_v1.exceptions import ApiException

# import models into sdk package
from synctera_client_v1.models.account_summary import AccountSummary
from synctera_client_v1.models.account_summary_balance_ceiling import AccountSummaryBalanceCeiling
from synctera_client_v1.models.account_summary_balance_floor import AccountSummaryBalanceFloor
from synctera_client_v1.models.ach_transaction import AchTransaction
from synctera_client_v1.models.ach_transaction_all_of import AchTransactionAllOf
from synctera_client_v1.models.ach_transaction_data import AchTransactionData
from synctera_client_v1.models.ach_transaction_subtypes import AchTransactionSubtypes
from synctera_client_v1.models.address import Address
from synctera_client_v1.models.address1 import Address1
from synctera_client_v1.models.adverse_action_reason_code import AdverseActionReasonCode
from synctera_client_v1.models.all_application_status import AllApplicationStatus
from synctera_client_v1.models.all_transaction_subtypes import AllTransactionSubtypes
from synctera_client_v1.models.apple_digital_wallet_provision_request import AppleDigitalWalletProvisionRequest
from synctera_client_v1.models.apple_digital_wallet_provision_response import AppleDigitalWalletProvisionResponse
from synctera_client_v1.models.applicant import Applicant
from synctera_client_v1.models.application import Application
from synctera_client_v1.models.application_list import ApplicationList
from synctera_client_v1.models.application_list_all_of import ApplicationListAllOf
from synctera_client_v1.models.application_patch import ApplicationPatch
from synctera_client_v1.models.application_response import ApplicationResponse
from synctera_client_v1.models.application_type import ApplicationType
from synctera_client_v1.models.auth_request_model import AuthRequestModel
from synctera_client_v1.models.authorization_advice_model import AuthorizationAdviceModel
from synctera_client_v1.models.balance_inquiry_request_model import BalanceInquiryRequestModel
from synctera_client_v1.models.base_card import BaseCard
from synctera_client_v1.models.base_card_all_of import BaseCardAllOf
from synctera_client_v1.models.base_statement import BaseStatement
from synctera_client_v1.models.base_statement_all_of import BaseStatementAllOf
from synctera_client_v1.models.base_transaction import BaseTransaction
from synctera_client_v1.models.base_transaction_decline import BaseTransactionDecline
from synctera_client_v1.models.billing_address import BillingAddress
from synctera_client_v1.models.business import Business
from synctera_client_v1.models.card_acceptor_model import CardAcceptorModel
from synctera_client_v1.models.card_activation_request import CardActivationRequest
from synctera_client_v1.models.card_brand import CardBrand
from synctera_client_v1.models.card_change import CardChange
from synctera_client_v1.models.card_change_state import CardChangeState
from synctera_client_v1.models.card_changes_list import CardChangesList
from synctera_client_v1.models.card_changes_list_all_of import CardChangesListAllOf
from synctera_client_v1.models.card_edit_request import CardEditRequest
from synctera_client_v1.models.card_format import CardFormat
from synctera_client_v1.models.card_fulfillment_status import CardFulfillmentStatus
from synctera_client_v1.models.card_image_details import CardImageDetails
from synctera_client_v1.models.card_image_details_list import CardImageDetailsList
from synctera_client_v1.models.card_image_details_list_all_of import CardImageDetailsListAllOf
from synctera_client_v1.models.card_image_mode import CardImageMode
from synctera_client_v1.models.card_image_rejection_reason import CardImageRejectionReason
from synctera_client_v1.models.card_image_status import CardImageStatus
from synctera_client_v1.models.card_issuance_request import CardIssuanceRequest
from synctera_client_v1.models.card_list_response import CardListResponse
from synctera_client_v1.models.card_list_response_all_of import CardListResponseAllOf
from synctera_client_v1.models.card_options import CardOptions
from synctera_client_v1.models.card_pin_status import CardPinStatus
from synctera_client_v1.models.card_product import CardProduct
from synctera_client_v1.models.card_product_all_of import CardProductAllOf
from synctera_client_v1.models.card_product_list_response import CardProductListResponse
from synctera_client_v1.models.card_product_list_response_all_of import CardProductListResponseAllOf
from synctera_client_v1.models.card_product_response import CardProductResponse
from synctera_client_v1.models.card_product_response_all_of import CardProductResponseAllOf
from synctera_client_v1.models.card_response import CardResponse
from synctera_client_v1.models.card_status import CardStatus
from synctera_client_v1.models.card_status_object import CardStatusObject
from synctera_client_v1.models.card_status_reason_code import CardStatusReasonCode
from synctera_client_v1.models.card_status_request import CardStatusRequest
from synctera_client_v1.models.card_transaction import CardTransaction
from synctera_client_v1.models.card_transaction_all_of import CardTransactionAllOf
from synctera_client_v1.models.card_transaction_data import CardTransactionData
from synctera_client_v1.models.card_transaction_data_currency_conversion import CardTransactionDataCurrencyConversion
from synctera_client_v1.models.card_transaction_data_merchant import CardTransactionDataMerchant
from synctera_client_v1.models.card_transaction_data_pos import CardTransactionDataPos
from synctera_client_v1.models.card_transaction_subtypes import CardTransactionSubtypes
from synctera_client_v1.models.card_type import CardType
from synctera_client_v1.models.card_widget_url_response import CardWidgetUrlResponse
from synctera_client_v1.models.cash_transaction import CashTransaction
from synctera_client_v1.models.cash_transaction_all_of import CashTransactionAllOf
from synctera_client_v1.models.cash_transaction_data import CashTransactionData
from synctera_client_v1.models.cash_transaction_subtypes import CashTransactionSubtypes
from synctera_client_v1.models.change_channel import ChangeChannel
from synctera_client_v1.models.change_type import ChangeType
from synctera_client_v1.models.charge_secured_statement import ChargeSecuredStatement
from synctera_client_v1.models.charge_secured_statement_all_of import ChargeSecuredStatementAllOf
from synctera_client_v1.models.check_transaction import CheckTransaction
from synctera_client_v1.models.check_transaction_all_of import CheckTransactionAllOf
from synctera_client_v1.models.check_transaction_data import CheckTransactionData
from synctera_client_v1.models.check_transaction_subtypes import CheckTransactionSubtypes
from synctera_client_v1.models.clearing_model import ClearingModel
from synctera_client_v1.models.client_token import ClientToken
from synctera_client_v1.models.create_card_image_request import CreateCardImageRequest
from synctera_client_v1.models.create_gateway_request import CreateGatewayRequest
from synctera_client_v1.models.create_transfer_response import CreateTransferResponse
from synctera_client_v1.models.create_transfer_response_pull import CreateTransferResponsePull
from synctera_client_v1.models.create_transfer_response_pull_all_of import CreateTransferResponsePullAllOf
from synctera_client_v1.models.create_transfer_response_push import CreateTransferResponsePush
from synctera_client_v1.models.create_transfer_response_push_all_of import CreateTransferResponsePushAllOf
from synctera_client_v1.models.create_transfer_type import CreateTransferType
from synctera_client_v1.models.credit_application import CreditApplication
from synctera_client_v1.models.credit_application_patch import CreditApplicationPatch
from synctera_client_v1.models.credit_application_response import CreditApplicationResponse
from synctera_client_v1.models.credit_application_response_all_of import CreditApplicationResponseAllOf
from synctera_client_v1.models.credit_application_status import CreditApplicationStatus
from synctera_client_v1.models.credit_card import CreditCard
from synctera_client_v1.models.credit_card_request_details import CreditCardRequestDetails
from synctera_client_v1.models.credit_card_response import CreditCardResponse
from synctera_client_v1.models.credit_card_response_details import CreditCardResponseDetails
from synctera_client_v1.models.credit_score import CreditScore
from synctera_client_v1.models.credit_summary import CreditSummary
from synctera_client_v1.models.customer_service_details import CustomerServiceDetails
from synctera_client_v1.models.dc_sign import DcSign
from synctera_client_v1.models.debit_card import DebitCard
from synctera_client_v1.models.debit_card_request_details import DebitCardRequestDetails
from synctera_client_v1.models.debit_card_response import DebitCardResponse
from synctera_client_v1.models.debit_card_response_details import DebitCardResponseDetails
from synctera_client_v1.models.decline_info import DeclineInfo
from synctera_client_v1.models.delete_response import DeleteResponse
from synctera_client_v1.models.depository_statement import DepositoryStatement
from synctera_client_v1.models.depository_statement_all_of import DepositoryStatementAllOf
from synctera_client_v1.models.device_type import DeviceType
from synctera_client_v1.models.digital_wallet_token_address_verification import DigitalWalletTokenAddressVerification
from synctera_client_v1.models.digital_wallet_token_edit_request import DigitalWalletTokenEditRequest
from synctera_client_v1.models.digital_wallet_token_response import DigitalWalletTokenResponse
from synctera_client_v1.models.digital_wallet_token_state import DigitalWalletTokenState
from synctera_client_v1.models.digital_wallet_tokenization import DigitalWalletTokenization
from synctera_client_v1.models.emboss_name import EmbossName
from synctera_client_v1.models.enhanced_raw_details import EnhancedRawDetails
from synctera_client_v1.models.enhanced_transaction_data import EnhancedTransactionData
from synctera_client_v1.models.enhanced_transaction_data1 import EnhancedTransactionData1
from synctera_client_v1.models.enhanced_transaction_data1_enhanced_raw_inner import EnhancedTransactionData1EnhancedRawInner
from synctera_client_v1.models.error import Error
from synctera_client_v1.models.event import Event
from synctera_client_v1.models.event_list import EventList
from synctera_client_v1.models.event_list_all_of import EventListAllOf
from synctera_client_v1.models.event_list_event import EventListEvent
from synctera_client_v1.models.event_trigger import EventTrigger
from synctera_client_v1.models.event_type import EventType
from synctera_client_v1.models.event_type_explicit import EventTypeExplicit
from synctera_client_v1.models.event_type_wildcard import EventTypeWildcard
from synctera_client_v1.models.external_card_list_response import ExternalCardListResponse
from synctera_client_v1.models.external_card_list_response_all_of import ExternalCardListResponseAllOf
from synctera_client_v1.models.external_card_request import ExternalCardRequest
from synctera_client_v1.models.external_card_response import ExternalCardResponse
from synctera_client_v1.models.external_card_status import ExternalCardStatus
from synctera_client_v1.models.external_card_transaction import ExternalCardTransaction
from synctera_client_v1.models.external_card_transaction_all_of import ExternalCardTransactionAllOf
from synctera_client_v1.models.external_card_transaction_subtypes import ExternalCardTransactionSubtypes
from synctera_client_v1.models.external_card_update_request import ExternalCardUpdateRequest
from synctera_client_v1.models.external_card_verifications import ExternalCardVerifications
from synctera_client_v1.models.financial_institution import FinancialInstitution
from synctera_client_v1.models.financial_request_model import FinancialRequestModel
from synctera_client_v1.models.financial_status import FinancialStatus
from synctera_client_v1.models.form import Form
from synctera_client_v1.models.fraud_risk_data import FraudRiskData
from synctera_client_v1.models.fulfillment_details import FulfillmentDetails
from synctera_client_v1.models.gateway_list_response import GatewayListResponse
from synctera_client_v1.models.gateway_list_response_all_of import GatewayListResponseAllOf
from synctera_client_v1.models.gateway_response import GatewayResponse
from synctera_client_v1.models.gateway_standin import GatewayStandin
from synctera_client_v1.models.get_card_barcode200_response import GetCardBarcode200Response
from synctera_client_v1.models.google_digital_wallet_provision_request import GoogleDigitalWalletProvisionRequest
from synctera_client_v1.models.google_digital_wallet_provision_response import GoogleDigitalWalletProvisionResponse
from synctera_client_v1.models.in_app_provisioning import InAppProvisioning
from synctera_client_v1.models.income import Income
from synctera_client_v1.models.income_base import IncomeBase
from synctera_client_v1.models.income_base_frequency import IncomeBaseFrequency
from synctera_client_v1.models.income_base_without_other_details import IncomeBaseWithoutOtherDetails
from synctera_client_v1.models.income_type import IncomeType
from synctera_client_v1.models.income_type_with_other_details import IncomeTypeWithOtherDetails
from synctera_client_v1.models.income_type_with_other_details_all_of import IncomeTypeWithOtherDetailsAllOf
from synctera_client_v1.models.internal_transfer_transaction import InternalTransferTransaction
from synctera_client_v1.models.internal_transfer_transaction_all_of import InternalTransferTransactionAllOf
from synctera_client_v1.models.internal_transfer_transaction_data import InternalTransferTransactionData
from synctera_client_v1.models.internal_transfer_transaction_subtypes import InternalTransferTransactionSubtypes
from synctera_client_v1.models.line_of_credit_statement import LineOfCreditStatement
from synctera_client_v1.models.line_of_credit_statement_all_of import LineOfCreditStatementAllOf
from synctera_client_v1.models.manual_entry import ManualEntry
from synctera_client_v1.models.merchant import Merchant
from synctera_client_v1.models.network_fee_model import NetworkFeeModel
from synctera_client_v1.models.original_credit_request_model import OriginalCreditRequestModel
from synctera_client_v1.models.original_credit_sender_data import OriginalCreditSenderData
from synctera_client_v1.models.paginated_response import PaginatedResponse
from synctera_client_v1.models.paginated_transactions import PaginatedTransactions
from synctera_client_v1.models.payment_sub_type import PaymentSubType
from synctera_client_v1.models.payment_type import PaymentType
from synctera_client_v1.models.person import Person
from synctera_client_v1.models.physical_card import PhysicalCard
from synctera_client_v1.models.physical_card_all_of import PhysicalCardAllOf
from synctera_client_v1.models.physical_card_format import PhysicalCardFormat
from synctera_client_v1.models.physical_card_plus_status import PhysicalCardPlusStatus
from synctera_client_v1.models.physical_credit_card_issuance_request import PhysicalCreditCardIssuanceRequest
from synctera_client_v1.models.physical_credit_card_response import PhysicalCreditCardResponse
from synctera_client_v1.models.physical_credit_card_response_all_of import PhysicalCreditCardResponseAllOf
from synctera_client_v1.models.physical_debit_card_issuance_request import PhysicalDebitCardIssuanceRequest
from synctera_client_v1.models.physical_debit_card_response import PhysicalDebitCardResponse
from synctera_client_v1.models.physical_debit_card_response_status import PhysicalDebitCardResponseStatus
from synctera_client_v1.models.physical_debit_card_response_status_all_of import PhysicalDebitCardResponseStatusAllOf
from synctera_client_v1.models.provisioning_controls import ProvisioningControls
from synctera_client_v1.models.push_tokenize_request_data import PushTokenizeRequestData
from synctera_client_v1.models.recipient_name import RecipientName
from synctera_client_v1.models.response_history_item import ResponseHistoryItem
from synctera_client_v1.models.restricted_application import RestrictedApplication
from synctera_client_v1.models.restricted_application_details import RestrictedApplicationDetails
from synctera_client_v1.models.restricted_application_document import RestrictedApplicationDocument
from synctera_client_v1.models.restricted_application_item import RestrictedApplicationItem
from synctera_client_v1.models.restricted_application_item_type import RestrictedApplicationItemType
from synctera_client_v1.models.restricted_application_page import RestrictedApplicationPage
from synctera_client_v1.models.restricted_application_patch import RestrictedApplicationPatch
from synctera_client_v1.models.restricted_application_question import RestrictedApplicationQuestion
from synctera_client_v1.models.restricted_application_response import RestrictedApplicationResponse
from synctera_client_v1.models.restricted_application_response_all_of import RestrictedApplicationResponseAllOf
from synctera_client_v1.models.restricted_application_section import RestrictedApplicationSection
from synctera_client_v1.models.restricted_application_status import RestrictedApplicationStatus
from synctera_client_v1.models.reversal_model import ReversalModel
from synctera_client_v1.models.review_details import ReviewDetails
from synctera_client_v1.models.saving_summary import SavingSummary
from synctera_client_v1.models.security_account_summary import SecurityAccountSummary
from synctera_client_v1.models.shipping import Shipping
from synctera_client_v1.models.shipping_address import ShippingAddress
from synctera_client_v1.models.simulate_card_fulfillment import SimulateCardFulfillment
from synctera_client_v1.models.spend_control import SpendControl
from synctera_client_v1.models.spend_control_creation_request import SpendControlCreationRequest
from synctera_client_v1.models.spend_control_direction import SpendControlDirection
from synctera_client_v1.models.spend_control_response import SpendControlResponse
from synctera_client_v1.models.spend_control_response_all_of import SpendControlResponseAllOf
from synctera_client_v1.models.spend_control_response_list import SpendControlResponseList
from synctera_client_v1.models.spend_control_response_list_all_of import SpendControlResponseListAllOf
from synctera_client_v1.models.spend_control_rolling_window_days import SpendControlRollingWindowDays
from synctera_client_v1.models.spend_control_rolling_window_days_all_of import SpendControlRollingWindowDaysAllOf
from synctera_client_v1.models.spend_control_single_transaction import SpendControlSingleTransaction
from synctera_client_v1.models.spend_control_time_range import SpendControlTimeRange
from synctera_client_v1.models.spend_control_time_range_type import SpendControlTimeRangeType
from synctera_client_v1.models.spend_control_update_request import SpendControlUpdateRequest
from synctera_client_v1.models.statement import Statement
from synctera_client_v1.models.statement_list import StatementList
from synctera_client_v1.models.statement_list_all_of import StatementListAllOf
from synctera_client_v1.models.statement_summary import StatementSummary
from synctera_client_v1.models.statement_type import StatementType
from synctera_client_v1.models.three_ds_policy import ThreeDsPolicy
from synctera_client_v1.models.token_list import TokenList
from synctera_client_v1.models.token_list_response import TokenListResponse
from synctera_client_v1.models.transaction import Transaction
from synctera_client_v1.models.transaction1 import Transaction1
from synctera_client_v1.models.transaction_data_v0 import TransactionDataV0
from synctera_client_v1.models.transaction_line_v0 import TransactionLineV0
from synctera_client_v1.models.transaction_list import TransactionList
from synctera_client_v1.models.transaction_list_all_of import TransactionListAllOf
from synctera_client_v1.models.transaction_options import TransactionOptions
from synctera_client_v1.models.transaction_statuses import TransactionStatuses
from synctera_client_v1.models.transaction_types import TransactionTypes
from synctera_client_v1.models.transaction_v0 import TransactionV0
from synctera_client_v1.models.transfer_list_response import TransferListResponse
from synctera_client_v1.models.transfer_list_response_all_of import TransferListResponseAllOf
from synctera_client_v1.models.transfer_request import TransferRequest
from synctera_client_v1.models.transfer_request_base import TransferRequestBase
from synctera_client_v1.models.transfer_request_pull import TransferRequestPull
from synctera_client_v1.models.transfer_request_pull_all_of import TransferRequestPullAllOf
from synctera_client_v1.models.transfer_request_push import TransferRequestPush
from synctera_client_v1.models.transfer_request_push_all_of import TransferRequestPushAllOf
from synctera_client_v1.models.transfer_response import TransferResponse
from synctera_client_v1.models.transfer_response_base import TransferResponseBase
from synctera_client_v1.models.transfer_response_pull import TransferResponsePull
from synctera_client_v1.models.transfer_response_pull_all_of import TransferResponsePullAllOf
from synctera_client_v1.models.transfer_response_push import TransferResponsePush
from synctera_client_v1.models.transfer_response_push_all_of import TransferResponsePushAllOf
from synctera_client_v1.models.transfer_reversal_request import TransferReversalRequest
from synctera_client_v1.models.transfer_type import TransferType
from synctera_client_v1.models.transfer_type_request import TransferTypeRequest
from synctera_client_v1.models.trigger_event_request import TriggerEventRequest
from synctera_client_v1.models.txn_enhancer import TxnEnhancer
from synctera_client_v1.models.update_card_image_request import UpdateCardImageRequest
from synctera_client_v1.models.update_gateway_request import UpdateGatewayRequest
from synctera_client_v1.models.virtual_card import VirtualCard
from synctera_client_v1.models.virtual_card_plus_status import VirtualCardPlusStatus
from synctera_client_v1.models.virtual_credit_card_issuance_request import VirtualCreditCardIssuanceRequest
from synctera_client_v1.models.virtual_credit_card_response import VirtualCreditCardResponse
from synctera_client_v1.models.virtual_credit_card_response_all_of import VirtualCreditCardResponseAllOf
from synctera_client_v1.models.virtual_debit_card_issuance_request import VirtualDebitCardIssuanceRequest
from synctera_client_v1.models.virtual_debit_card_response import VirtualDebitCardResponse
from synctera_client_v1.models.virtual_debit_card_response_status import VirtualDebitCardResponseStatus
from synctera_client_v1.models.wallet_provider_card_on_file import WalletProviderCardOnFile
from synctera_client_v1.models.webhook import Webhook
from synctera_client_v1.models.webhook_list import WebhookList
from synctera_client_v1.models.webhook_list_all_of import WebhookListAllOf
from synctera_client_v1.models.webhook_request_object import WebhookRequestObject
from synctera_client_v1.models.widget_type import WidgetType
from synctera_client_v1.models.wire_transaction import WireTransaction
from synctera_client_v1.models.wire_transaction_all_of import WireTransactionAllOf
from synctera_client_v1.models.wire_transaction_data import WireTransactionData
from synctera_client_v1.models.wire_transaction_subtypes import WireTransactionSubtypes
from synctera_client_v1.models.withdrawal_request_model import WithdrawalRequestModel



# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr

class AchTransactionData(BaseModel):
    """
    Transaction metadata specific to transactions with type `ACH`
    """
    description: Optional[StrictStr] = Field(None, description="The ACH company entry description")
    file_name: Optional[StrictStr] = Field(None, description="For outgoing ACH, the name of the file the entry went out in. For incoming ACH, the name of the file the entry came from. The value will be omitted for outgoing payments that have not yet been written into a file.")
    id: StrictStr = Field(..., description="The ACH payment uuid (used in `/v0/ach` endpoint)")
    identification_number: Optional[StrictStr] = Field(None, description="An internal identification number used to uniquely identify the ACH entry. For Person-to-Person WEB entries, this should name of the originator.")
    originating_account_id: Optional[StrictStr] = Field(None, description="The uuid of the account originating the ACH. This will be a customer account uuid for outgoing ACH, and omitted for incoming ACH.")
    originating_company_name: Optional[StrictStr] = Field(None, description="The name of the company originating the ACH entry.")
    originating_routing_number: Optional[StrictStr] = Field(None, description="The routing number of the originating DFI.")
    originator_name: Optional[StrictStr] = Field(None, description="The name of the originator, if available.")
    receiving_account_id: Optional[StrictStr] = Field(None, description="The uuid of the account receiving the ACH entry. In the case of an outgoing ACH, this will be an external_account uuid. For incoming ACH, this will be an account uuid.")
    recipient_name: Optional[StrictStr] = Field(None, description="The name of the recipient according to the ACH entry. This should map to the ACH `Individual Name` field.")
    return_code: Optional[StrictStr] = Field(None, description="The ACH return code, if this transaction was a return")
    trace_number: Optional[StrictStr] = Field(None, description="The ACH trace number associated with the transaction")
    __properties = ["description", "file_name", "id", "identification_number", "originating_account_id", "originating_company_name", "originating_routing_number", "originator_name", "receiving_account_id", "recipient_name", "return_code", "trace_number"]

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AchTransactionData:
        """Create an instance of AchTransactionData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AchTransactionData:
        """Create an instance of AchTransactionData from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return AchTransactionData.parse_obj(obj)

        _obj = AchTransactionData.parse_obj({
            "description": obj.get("description"),
            "file_name": obj.get("file_name"),
            "id": obj.get("id"),
            "identification_number": obj.get("identification_number"),
            "originating_account_id": obj.get("originating_account_id"),
            "originating_company_name": obj.get("originating_company_name"),
            "originating_routing_number": obj.get("originating_routing_number"),
            "originator_name": obj.get("originator_name"),
            "receiving_account_id": obj.get("receiving_account_id"),
            "recipient_name": obj.get("recipient_name"),
            "return_code": obj.get("return_code"),
            "trace_number": obj.get("trace_number")
        })
        return _obj




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 1.41.3
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import json
import pprint
import re  # noqa: F401

from typing import Optional
from pydantic import BaseModel, Field, StrictStr, ValidationError, validator
from synctera_client_v1.models.ach_transaction_subtypes import AchTransactionSubtypes
from synctera_client_v1.models.card_transaction_subtypes import CardTransactionSubtypes
from synctera_client_v1.models.cash_transaction_subtypes import CashTransactionSubtypes
from synctera_client_v1.models.check_transaction_subtypes import CheckTransactionSubtypes
from synctera_client_v1.models.external_card_transaction_subtypes import ExternalCardTransactionSubtypes
from synctera_client_v1.models.internal_transfer_transaction_subtypes import InternalTransferTransactionSubtypes
from synctera_client_v1.models.wire_transaction_subtypes import WireTransactionSubtypes
from typing import Any, List
from pydantic import StrictStr, Field

ALLTRANSACTIONSUBTYPES_ANY_OF_SCHEMAS = ["AchTransactionSubtypes", "CardTransactionSubtypes", "CashTransactionSubtypes", "CheckTransactionSubtypes", "ExternalCardTransactionSubtypes", "InternalTransferTransactionSubtypes", "WireTransactionSubtypes"]

class AllTransactionSubtypes(BaseModel):
    """
    The set of all possible transaction subtypes
    """

    # data type: ExternalCardTransactionSubtypes
    anyof_schema_1_validator: Optional[ExternalCardTransactionSubtypes] = None
    # data type: WireTransactionSubtypes
    anyof_schema_2_validator: Optional[WireTransactionSubtypes] = None
    # data type: CardTransactionSubtypes
    anyof_schema_3_validator: Optional[CardTransactionSubtypes] = None
    # data type: CheckTransactionSubtypes
    anyof_schema_4_validator: Optional[CheckTransactionSubtypes] = None
    # data type: InternalTransferTransactionSubtypes
    anyof_schema_5_validator: Optional[InternalTransferTransactionSubtypes] = None
    # data type: CashTransactionSubtypes
    anyof_schema_6_validator: Optional[CashTransactionSubtypes] = None
    # data type: AchTransactionSubtypes
    anyof_schema_7_validator: Optional[AchTransactionSubtypes] = None
    actual_instance: Any
    any_of_schemas: List[str] = Field(ALLTRANSACTIONSUBTYPES_ANY_OF_SCHEMAS, const=True)

    class Config:
        validate_assignment = True

    def __init__(self, *args, **kwargs):
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @validator('actual_instance')
    def actual_instance_must_validate_anyof(cls, v):
        instance = AllTransactionSubtypes.construct()
        error_messages = []
        # validate data type: ExternalCardTransactionSubtypes
        if not isinstance(v, ExternalCardTransactionSubtypes):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ExternalCardTransactionSubtypes`")
        else:
            return v

        # validate data type: WireTransactionSubtypes
        if not isinstance(v, WireTransactionSubtypes):
            error_messages.append(f"Error! Input type `{type(v)}` is not `WireTransactionSubtypes`")
        else:
            return v

        # validate data type: CardTransactionSubtypes
        if not isinstance(v, CardTransactionSubtypes):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CardTransactionSubtypes`")
        else:
            return v

        # validate data type: CheckTransactionSubtypes
        if not isinstance(v, CheckTransactionSubtypes):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CheckTransactionSubtypes`")
        else:
            return v

        # validate data type: InternalTransferTransactionSubtypes
        if not isinstance(v, InternalTransferTransactionSubtypes):
            error_messages.append(f"Error! Input type `{type(v)}` is not `InternalTransferTransactionSubtypes`")
        else:
            return v

        # validate data type: CashTransactionSubtypes
        if not isinstance(v, CashTransactionSubtypes):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CashTransactionSubtypes`")
        else:
            return v

        # validate data type: AchTransactionSubtypes
        if not isinstance(v, AchTransactionSubtypes):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AchTransactionSubtypes`")
        else:
            return v

        if error_messages:
            # no match
            raise ValueError("No match found when setting the actual_instance in AllTransactionSubtypes with anyOf schemas: AchTransactionSubtypes, CardTransactionSubtypes, CashTransactionSubtypes, CheckTransactionSubtypes, ExternalCardTransactionSubtypes, InternalTransferTransactionSubtypes, WireTransactionSubtypes. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: dict) -> AllTransactionSubtypes:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> AllTransactionSubtypes:
        """Returns the object represented by the json string"""
        instance = AllTransactionSubtypes.construct()
        error_messages = []
        # anyof_schema_1_validator: Optional[ExternalCardTransactionSubtypes] = None
        try:
            instance.actual_instance = ExternalCardTransactionSubtypes.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
             error_messages.append(str(e))
        # anyof_schema_2_validator: Optional[WireTransactionSubtypes] = None
        try:
            instance.actual_instance = WireTransactionSubtypes.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
             error_messages.append(str(e))
        # anyof_schema_3_validator: Optional[CardTransactionSubtypes] = None
        try:
            instance.actual_instance = CardTransactionSubtypes.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
             error_messages.append(str(e))
        # anyof_schema_4_validator: Optional[CheckTransactionSubtypes] = None
        try:
            instance.actual_instance = CheckTransactionSubtypes.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
             error_messages.append(str(e))
        # anyof_schema_5_validator: Optional[InternalTransferTransactionSubtypes] = None
        try:
            instance.actual_instance = InternalTransferTransactionSubtypes.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
             error_messages.append(str(e))
        # anyof_schema_6_validator: Optional[CashTransactionSubtypes] = None
        try:
            instance.actual_instance = CashTransactionSubtypes.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
             error_messages.append(str(e))
        # anyof_schema_7_validator: Optional[AchTransactionSubtypes] = None
        try:
            instance.actual_instance = AchTransactionSubtypes.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
             error_messages.append(str(e))

        if error_messages:
            # no match
            raise ValueError("No match found when deserializing the JSON string into AllTransactionSubtypes with anyOf schemas: AchTransactionSubtypes, CardTransactionSubtypes, CashTransactionSubtypes, CheckTransactionSubtypes, ExternalCardTransactionSubtypes, InternalTransferTransactionSubtypes, WireTransactionSubtypes. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        to_json = getattr(self.actual_instance, "to_json", None)
        if callable(to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> dict:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        to_json = getattr(self.actual_instance, "to_json", None)
        if callable(to_json):
            return self.actual_instance.to_dict()
        else:
            return json.dumps(self.actual_instance)

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.dict())

